temp = 1;
% Log Analysis Toggle    
    logAnalysisToggle = 1;          % 1 for log analysis, 0 for normal operation

% Define Simulink Runtime (if logAnalysisToggle is selected, this will be
% automatically set based on the log files time)
    runtime = 20;
    
% Model Parameters
    m = 1.244;                      % Quadrotor + battery mass
    g = 9.81;                       % Acceleration of gravity
    Jxx = 0.0130;                   % Quadrotor and battery motor of inertia around bx (pitch)
    Jyy = 0.0140;                   % Quadrotor and battery motor of inertia around by (roll)
    Jzz = 0.0285;                   % Quadrotor and battery motor of inertia around bz (yaw)
    Jreq = 4.2012e-05;              % Rotor and motor moment of inertia around axis of rotation 
    Kt = 8.6519e-6;                 % Rotor thrust constant
    Kh = 0;                         % Rotor in-plane drag constant
    Kd = 1.0317e-7;                 % Rotor drag constant
    rhx = 0.16;                     % X-axis distance from center of mass to a rotor hub
    rhy = 0.16;                     % Y-axis distance from center of mass to a rotor hub
    rhz = 0.03;                     % Z-axis distance from center of mass to a rotor hub
    Rm = 0.2308;                    % Motor resistance
    Kq = 96.3422;                   % Motor torque constant
    Kv = 96.3422;                   % Motor back emf constant
    If = 0.511;                     % Motor internal friction current
    Pmin = 1e5;                     % Minimum zybo output duty cycle command
    Pmax = 2e5;                     % Maximum zybo output duty cycle command
    Tc = 0.04;                      % Camera system sampling period
    Tq = 0.005;                     % Quad sampling period
    tau_c = 0;                      % Camera system total latency
    Vb = 11.1;                      % Nominal battery voltage (V)
    x_controlled_o = 0;             % Equilibrium lateral controller output
    y_controlled_o = 0;             % Equilibrium longitudinal controller output
    yaw_controlled_o = 0;           % Equilibrium yaw controller output
    
    % Determine Initial Conditions
    
    % Equilibrium rotor speeds
    omega0_o = sqrt((m*g)/(4*Kt));
    omega1_o = sqrt((m*g)/(4*Kt));
    omega2_o = sqrt((m*g)/(4*Kt));
    omega3_o = sqrt((m*g)/(4*Kt));
    
    % Equilibrium height controller output
    height_controlled_o = (((Rm*If + ...
    + (((omega0_o * 2 * Rm * Kv * Kq  ...
    * Kd + 1)^2) - 1)/(4* Rm*Kv^2*Kq ...
    *Kd))/Vb)*(Pmax- Pmin)+Pmin);

    % Equilibrium positions
    x_o = 0;
    y_o = 0;
    z_o = 0;

    % Equilibrium velocities
    x_vel_o = 0;
    y_vel_o = 0;
    z_vel_o = 0;
    
    % Equilibrium angles
    roll_o = 0;
    pitch_o = 0;
    yaw_o = 0;
    
    % Equilibrium angular rates
    rollrate_o = 0;
    pitchrate_o = 0;
    yawrate_o = 0;
    
% Import Data and determine errors
if logAnalysisToggle == 1 && temp == 0
    % Import Data to Workspace
    data = importdata('loggingAnalysis/logFiles/logData.csv');

    % Set up time vector
    time = data.data(:, 1);
    runtime = max(time);
   
    % Determine x position error
    x_setpoint = data.data(:, 25);
    x_position = data.data(:, 20);
    x_error = timeseries(x_setpoint - x_position, time);

    % Determine y position error
    y_setpoint = data.data(:, 26);
    y_position = data.data(:, 21);
    y_error = timeseries(y_setpoint - y_position, time);

    % Determine z position error
    z_setpoint = data.data(:, 27);
    z_position = data.data(:, 22);
    z_error = timeseries(z_setpoint - z_position, time);

    % Determine pitch error
    pitch_setpoint = data.data(:, 9);
    pitch_value = data.data(:, 23);
    pitch_error = timeseries(pitch_setpoint - pitch_value, time);

    % Determine roll error
    roll_setpoint = data.data(:, 10);
    roll_value = data.data(:, 24);
    roll_error = timeseries(roll_setpoint - roll_value, time);

    % Determine yaw error
    yaw_setpoint = zeros(length(time), 1); % NEEDS UPDATED WHEN LOG FILE INCLUDES YAW SETPOINT
    yaw_value = data.data(:,10);%(length(time), 1); % NEEDS UPDATED WHEN LOG FILE INCLUDES YAW VALUE
    yaw_error = timeseries(yaw_setpoint - yaw_value, time);

    % Determine pitch rate error
    pitchrate_setpoint = data.data(:, 11);
    pitchrate_value = data.data(:, 6);
    pitchrate_error = timeseries(pitchrate_setpoint - pitchrate_value, time);

    % Determine roll rate error
    rollrate_setpoint = data.data(:, 12);
    rollrate_value = data.data(:, 5);
    rollrate_error = timeseries(rollrate_setpoint - rollrate_value, time);

    % Determine yaw rate error
    yawrate_setpoint = data.data(:, 13);
    yawrate_value = data.data(:, 7);
    yawrate_error = timeseries(yawrate_setpoint - yawrate_value, time);

    % Pull motor commands from log
    x_command = data.data(:, 14);
    y_command = data.data(:, 15);
    z_command = data.data(:, 8);
    yaw_command = data.data(:, 16);
    
    % Determine signal mix PWM values
    PWM0 = data.data(:, 28);
    PWM1 = data.data(:, 29);
    PWM2 = data.data(:, 30);
    PWM3 = data.data(:, 31);
    
    % Determine the initial height controller command
    % height_controlled_o = 1; % NEEDS UPDATED WHEN LOG FILES INCLUDE THROTTLE COMMAND
    
    % Determine the initial rotor speeds based on PWM inputs
    u_P0 = (PWM0(1) - Pmin) / (Pmax - Pmin);
    u_P1 = (PWM1(1) - Pmin) / (Pmax - Pmin);
    u_P2 = (PWM2(1) - Pmin) / (Pmax - Pmin);
    u_P3 = (PWM3(1) - Pmin) / (Pmax - Pmin);
    
    Vb_eff_0 = u_P0 * Vb;
    Vb_eff_1 = u_P1 * Vb;
    Vb_eff_2 = u_P2 * Vb;
    Vb_eff_3 = u_P3 * Vb;
    
    omega0_o = (-1 + sqrt(1 - 4*Rm*Kv*Kq*Kd*( Kv*Rm*If - Kv*Vb_eff_0))) / (2*Rm*Kv*Kq*Kd);
    omega1_o = (-1 + sqrt(1 - 4*Rm*Kv*Kq*Kd*( Kv*Rm*If - Kv*Vb_eff_1))) / (2*Rm*Kv*Kq*Kd);
    omega2_o = (-1 + sqrt(1 - 4*Rm*Kv*Kq*Kd*( Kv*Rm*If - Kv*Vb_eff_2))) / (2*Rm*Kv*Kq*Kd);
    omega3_o = (-1 + sqrt(1 - 4*Rm*Kv*Kq*Kd*( Kv*Rm*If - Kv*Vb_eff_3))) / (2*Rm*Kv*Kq*Kd);
    
    % Determine initial positions for x, y, and z
    x_o = x_position(1);
    y_o = y_position(1);
    z_o = z_position(1);
    
    % Determine initial velocity for x, y, and z
    x_vel_o = (x_position(2) - x_position(1)) / (time(2) - time(1));
    y_vel_o = (y_position(2) - y_position(1)) / (time(2) - time(1));
    z_vel_o = (z_position(2) - z_position(1)) / (time(2) - time(1));
    
    % Determine initial angles
    roll_o = roll_value(1);
    pitch_o = pitch_value(1);
    yaw_o = yaw_value(1);
    
    % Determine initial angular rates
    rollrate_o = rollrate_value(1);
    pitchrate_o = pitchrate_value(1);
    yawrate_o = yawrate_value(1);

elseif logAnalysisToggle == 1 && temp == 1
    %%%%%% Commented out section until logging is .txt file based %%%%%%
    % FNAME
    % if you know the name of the log file that you want to parse, set the it
    % here only if it is in the working directory. Otherwise, you may leave it
    % blank. You will be able to choose the file to parse through an explorer
    % window.
    %
    %fname = 'sampleLogFile.txt';  
    fname = '';
    fpath = '';
    
    if(isempty(fname))
        [fname, fpath] = uigetfile('.txt','Select log file');
    end
    
    params.file.name = fname;               % file name only
    params.file.path = fpath;               % file path only
    params.file.pathName = [fpath fname];   % file path with file name
    
    [dataStruct, headers] = parse_log_model(params.file.pathName);
    
    time = dataStruct.Time.data;
    runtime = max(time);
 
    % Determine x position error
    x_setpoint = dataStruct.X_Setpoint_Constant.data;
    x_position = dataStruct.VRPN_X_Constant.data;
    x_error = timeseries(x_setpoint - x_position, time);

    % Determine y position error
    y_setpoint = dataStruct.Y_Setpoint_Constant.data;
    y_position = dataStruct.VRPN_Y_Constant.data;
    y_error = timeseries(y_setpoint - y_position, time);

    % Determine z position error
    z_setpoint = dataStruct.Alt_Setpoint_Constant.data;
    z_position = dataStruct.VRPN_Alt_Constant.data;
    z_error = timeseries(z_setpoint - z_position, time);

    % Determine pitch error
    pitch_setpoint = dataStruct.X_pos_PID_Correction.data;
    pitch_value = dataStruct.VRPN_Pitch_Constant.data;
    pitch_error = timeseries(pitch_setpoint - pitch_value, time);

    % Determine roll error
    roll_setpoint = dataStruct.Y_pos_PID_Correction.data;
    roll_value = dataStruct.VRPN_Roll_Constant.data;
    roll_error = timeseries(roll_setpoint - roll_value, time);

    % Determine yaw error
    yaw_setpoint = zeros(length(time), 1); % NEEDS UPDATED WHEN LOG FILE INCLUDES YAW SETPOINT
    yaw_value = zeros(length(time), 1); % NEEDS UPDATED WHEN LOG FILE INCLUDES YAW VALUE
    yaw_error = timeseries(yaw_setpoint - yaw_value, time);

    % Determine pitch rate error
    pitchrate_setpoint = dataStruct.Pitch_PID_Correction.data;
    pitchrate_value = dataStruct.gyro_y.data;
    pitchrate_error = timeseries(pitchrate_setpoint - pitchrate_value, time);

    % Determine roll rate error
    rollrate_setpoint = dataStruct.Roll_PID_Correction.data;
    rollrate_value = dataStruct.gyro_x.data;
    rollrate_error = timeseries(rollrate_setpoint - rollrate_value, time);

    % Determine yaw rate error
    yawrate_setpoint = dataStruct.Yaw_PID_Correction.data;
    yawrate_value = dataStruct.gyro_z.data;
    yawrate_error = timeseries(yawrate_setpoint - yawrate_value, time);

    % Pull motor commands from log
    x_command = dataStruct.Pitch_Rate_PID_Correction.data;
    y_command = dataStruct.Roll_Rate_PID_Correction.data;
    z_command = dataStruct.Altitude_PID_Correction.data;
    yaw_command = dataStruct.Yaw_Rate_PID_Correction.data;
    
    % Determine signal mix PWM values
    PWM0 = dataStruct.Signal_Mixer_PWM_0.data;
    PWM1 = dataStruct.Signal_Mixer_PWM_1.data;
    PWM2 = dataStruct.Signal_Mixer_PWM_2.data;
    PWM3 = dataStruct.Signal_Mixer_PWM_3.data;
       
end


